@@ -6,7 +6,7 @@ function (Xdelta)
         I <-dim(Xdelta)[2]
         Z <- matrix(0,dim(Xdelta)[1],2*(I-1))
         Z[,1] <- (1-Xdelta[,1])
-        if(isTRUE(I>2))
+        if(I>2)
         {
             i <- 2
             for  (j in (2:(I-1)))
@@ -16,7 +16,7 @@ function (Xdelta)
             }
         }
         Z[,I] <- (1-Xdelta[,I])
-        if(isTRUE(I>2))
+        if(I>2)
         {
             i <- 1
             for  (j in ((I+1):(2*(I-1))))
