@@ -1,66 +1,68 @@
 "robustd.0" <- function(X, dfreq=FALSE, vt, vm="M0", vh=list("Chao"), va=2, neg=TRUE)
 {
 
-        I <- length(vt) # nombre de periodes primaires
-
 ############################################################################################################################################
 # Validation des arguments fournis en entre et changement de leur forme si ncessaire
 ############################################################################################################################################
 
         # Argument dfreq
-        if(!is.logical(dfreq)||!isTRUE(all.equal(length(dfreq),1))) stop("'dfreq' must be a logical object of length 1")
+        if(!is.logical(dfreq)||length(dfreq)!=1) stop("'dfreq' must be a logical object of length 1")
     
+            X <- as.matrix(X)
+            t <- if(dfreq) dim(X)[2]-1 else dim(X)[2]
+            I <- length(vt) # nombre de periodes primaires
+    
+        # Argument vt
+        if(t!=sum(na.rm=TRUE,vt))
+            stop("The number of columns in 'X' is not equal to the total number of capture occasions (sum of the 'vt' components)")
+        if (any((vt %% 1)!=0)) stop("The 'vt' components must be integers")
+
         # Argument X
-        X <- as.matrix(X)
         if (dfreq)
         {
-            if (any(X[,1:sum(vt)]!=1&X[,1:sum(vt)]!=0)) stop("Every columns of 'X' but the last one must contain only zeros and ones")
-            if (any((X[,sum(vt)+1] %% 1)!=0)) stop("The last column of 'X' must contain capture histories frequencies, therefore integers")
+            if (any(X[,1:t]!=1&X[,1:t]!=0)) stop("Every columns of 'X' but the last one must contain only zeros and ones")
+            if (any((X[,t+1] %% 1)!=0)) stop("The last column of 'X' must contain capture histories frequencies, therefore integers")
         } else {
             if(any(X!=1&X!=0)) stop("'X' must contain only zeros and ones")
         }
     
-        # Argument vt
-        if(!isTRUE(all.equal(ifelse(dfreq,dim(X)[2]-1,dim(X)[2]),sum(vt))))
-            stop("The number of columns in 'X' is not equal to the total number of capture occasions (sum of the 'vt' components)")
-        if (any((vt %% 1)!=0)) stop("The 'vt' components must be integers")
-    
         # Argument vm
-        if(!(isTRUE(all.equal(length(vt),length(vm)))||isTRUE(all.equal(length(vm),1)))) stop("'vm' must be of length 1 or have the same length than 'vt'")
+        if(!(length(vt)==length(vm)||length(vm)==1)) stop("'vm' must be of length 1 or have the same length than 'vt'")
         for (i in 1:length(vm))
         {
-            if(identical(vm[i],"Mt")||identical(vm[i],"Mth"))
-            stop("Models with a temporal effect cannot be adjusted")
-            if(!identical(vm[i],"none")&&!identical(vm[i],"M0")&&!identical(vm[i],"Mh"))
-            stop("'vm' components can only take the value 'none', 'M0', or 'Mh'")
-            if(identical(vm[i],"Mh")&&isTRUE(vt[i]<3))
-            stop("Mh models require at least 3 capture occasions")
+            if(vm[i]=="Mt"||vm[i]=="Mth")
+                stop("Models with a temporal effect can not be adjusted")
+            if(vm[i]!="none"&&vm[i]!="M0"&&vm[i]!="Mh")
+                stop("'vm' components can only take the value 'none', 'M0', or 'Mh'")
+            if(vm[i]=="Mh"&&vt[i]<3)
+                stop("Mh models require a least 3 capture occasions")
         }
-        if(identical(vm[1],"none")||identical(vm[length(vm)],"none")) stop("The 'no model' cannot be chosen for the first or the last period")
+        if(vm[1]=="none"||vm[length(vm)]=="none") stop("The 'no model' cannot be chosen for the first or the last period")
         # Forme
-        if (isTRUE(all.equal(length(vm),1))) vm <- rep(vm,length(vt))       
+        if (length(vm)==1) vm <- rep(vm,length(vt))       
         
     
         # Argument vh
-        nh <- length(vm[vm=="Mh"])
-        if (isTRUE(nh>0))
+        nh <- sum(na.rm=TRUE,vm=="Mh")
+        if (nh>0)
         { 
             vh <- as.list(vh)
-            if(!(isTRUE(all.equal(length(vh),nh))||isTRUE(all.equal(length(vh),1)))) stop("'vh' must be of length 1 or of length equal to the number of heterogenous models specified in 'vm'")
+            if(!(length(vh)==nh||length(vh)==1))
+                stop("'vh' must be of length 1 or of length equals the number of heterogenous models specified in 'vm'")
             for (i in 1:length(vh))
             {
-                if(!is.function(vh[[i]])&&!identical(vh[[i]],"Chao")&&!identical(vh[[i]],"Darroch")&&!identical(vh[[i]],"Poisson"))
-                stop("The 'vh' elements must be functions or charater strings taking the value 'Chao', 'Darroch' or 'Poisson'")
+                if(!is.function(vh[[i]])&&vh[[i]]!="Chao"&&vh[[i]]!="Darroch"&&vh[[i]]!="Poisson")
+                stop("The 'vh' elements must be functions or charater strings taking the values 'Chao', 'Darroch' or 'Poisson'")
             }
             # Forme
-            if (isTRUE(all.equal(length(vh),1))) vh <- rep(vh,nh)
+            if (length(vh)==1) vh <- rep(vh,nh)
             vht<-rep(NA,I)
             j<-1
             for (i in 1:I)
             { 
-                if (identical(vm[i],"Mh"))
+                if (vm[i]=="Mh")
                 {
-                    vht[[i]] <- vh[[j]]
+                    vht[i] <- vh[[j]]
                     j <- j+1
                 }
             }
@@ -68,18 +70,19 @@
         }
         
         # Argument va
-        nP <- length(vh[vh=="Poisson"])
-        if (isTRUE(nP>0))
+        nP <- sum(na.rm=TRUE,vh=="Poisson")
+        if (nP>0)
         {
-            if(!(isTRUE(all.equal(length(va),nP))||isTRUE(all.equal(length(va),1)))) stop("'va' must be of length 1 or of length equal to the number of Poisson heterogenous models specified in 'vh'")
+            if(!(length(va)==nP||length(va)==1))
+                stop("'va' must be of length 1 or of length equals the number of Poisson heterogenous models specified in 'vh'")
             if (!is.numeric(va)) stop("The 'va' components must be numeric values")
             # Forme
-            if (isTRUE(all.equal(length(va),1))) va <- rep(va,nP)
+            if (length(va)==1) va <- rep(va,nP)
             vat<-rep(1,I)
             j<-1
             for (i in 1:I)
             { 
-                if ((identical(vh[i],"Poisson")))
+                if (!is.na(vh[i])&&vh[i]=="Poisson")
                 {
                     vat[i] <- va[j]
                     j <- j+1
@@ -89,7 +92,7 @@
         }
        
         # Argument neg
-        if(!is.logical(neg)||!isTRUE(all.equal(length(neg),1))) stop("'neg' must be a logical object of length 1")
+        if(!is.logical(neg)||length(neg)!=1) stop("'neg' must be a logical object of length 1")
 
 
 
@@ -135,29 +138,29 @@
             indic <- as.vector(c(0,ifelse(param[2:(2*I-1)]<0,1,0)))
             for (i in 1:I)
             {
-                if ((identical(vm[i],"Mh")||identical(vm[i],"Mth"))&&identical(vh[[i]],"Chao")) {
-                    indic <- as.vector(c(indic,rep(0,Xw$nbparam[i]-vt[i]+2),ifelse(param[(2*I+sum(Xw$nbparam[1:i])-vt[i]+2):(2*I+sum(Xw$nbparam[1:i])-1)]<0,1,0)))
+                if ((vm[i]=="Mh"||vm[i]=="Mth")&&vh[i]=="Chao") {
+                    indic <- as.vector(c(indic,rep(0,Xw$nbparam[i]-vt[i]+2),ifelse(param[(2*I+sum(na.rm=TRUE,Xw$nbparam[1:i])-vt[i]+2):(2*I+sum(na.rm=TRUE,Xw$nbparam[1:i])-1)]<0,1,0)))
                 } else {
                     indic <- as.vector(c(indic,rep(0,Xw$nbparam[i])))
                 }
             }
-            while(isTRUE(sum(indic)>0)) # Rpter la boucle jusqu' ce qu'aucun gamma appropri ne soit ngatif
+            while(sum(na.rm=TRUE,indic)>0) # Rpter la boucle jusqu' ce qu'aucun gamma appropri ne soit ngatif
             {
                 # Dtermination de la position du premier gamma appropri ngatif
                 pos <- 1
-                while(isTRUE(all.equal(indic[pos],0))) pos <- pos + 1
+                while(indic[pos]==0) pos <- pos + 1
                 ppositions <- c(ppositions,pos)
                 # Retrait de la bonne colonne de mX et rajustement du modle
-                mX <- mX[,-(pos-sum(ppositions<pos))]
+                mX <- mX[,-(pos-sum(na.rm=TRUE,ppositions<pos))]
                 anaMrd <- glm(Y~offset(consp)+mX,family=poisson)        
                 # Ajout de zros dans le vecteur des paramtres loglinaires
                 positions <- sort(ppositions[-1])                
                 param <- c(anaMrd$coef[1:(positions[1]-1)],0)
-                if(isTRUE(length(positions)>1))
+                if(length(positions)>1)
                 {
                     for ( i in 2:length(positions))
                     {
-                        if(isTRUE(all.equal(positions[i],positions[i-1]+1))) {
+                        if(positions[i]==positions[i-1]+1) {
                             param <- c(param,0)
                         } else {
                             param <- c(param,anaMrd$coef[(positions[i-1]-i+2):(positions[i]-i)],0)
@@ -169,8 +172,8 @@
                 indic <- as.vector(c(0,ifelse(param[2:(2*I-1)]<0,1,0)))
                 for (i in 1:I)
                 {
-                    if ((identical(vm[[i]],"Mh")||identical(vm[[i]],"Mth"))&&identical(vh[[i]],"Chao")) {
-                        indic <- as.vector(c(indic,rep(0,Xw$nbparam[i]-vt[[i]]+2),ifelse(param[(2*I+sum(Xw$nbparam[1:i])-vt[[i]]+2):(2*I+sum(Xw$nbparam[1:i])-1)]<0,1,0)))
+                    if ((vm[[i]]=="Mh"||vm[[i]]=="Mth")&&vh[i]=="Chao") {
+                        indic <- as.vector(c(indic,rep(0,Xw$nbparam[i]-vt[[i]]+2),ifelse(param[(2*I+sum(na.rm=TRUE,Xw$nbparam[1:i])-vt[[i]]+2):(2*I+sum(na.rm=TRUE,Xw$nbparam[1:i])-1)]<0,1,0)))
                     } else {
                         indic <- as.vector(c(indic,rep(0,Xw$nbparam[i])))
                     }
@@ -186,10 +189,10 @@
 
         Inono <- length(vm[vm!="none"])
         idpemig <- 0
-        for (i in 2:(I-1)) if(!identical(vm[i],"none")) idpemig <- c(idpemig,i)
+        for (i in 2:(I-1)) if(vm[i]!="none") idpemig <- c(idpemig,i)
         idpemig <- idpemig[-1]
         
-        if(isTRUE(all.equal(Inono,3)))
+        if(Inono==3)
         {
             anaMrd2 <- NULL
             parap2 <- NULL
@@ -197,7 +200,7 @@
             mX3<-cbind(mX,Xdelta[,idpemig])
             anaMrd3 <- glm(Y~offset(consp)+mX3,family=poisson)
             parap3<-summary(anaMrd3)$coef[length(anaMrd3$coef),1:2]
-        } else if(isTRUE(Inono>3))
+        } else if(Inono>3)
             {
                 mX2<-cbind(mX,apply(Xdelta[,idpemig],1,sum))
                 anaMrd2 <- glm(Y~offset(consp)+mX2,family=poisson)
@@ -251,15 +254,15 @@
 # Matrice de variances-covariances des paramtres loglinaires #
 #--------------------------------------------------------------#
 
-        if(isTRUE(length(positions)>0))
+        if(length(positions)>0)
         {
             # Insertion de colonnes de zros
             varcovc <- cbind(summary(anaMrd)$cov.unscaled[,1:(positions[1]-1)],rep(0,dim(summary(anaMrd)$cov.unscaled)[1]))
-            if(isTRUE(length(positions)>1))
+            if(length(positions)>1)
             {
                 for ( i in 2:length(positions))
                 {
-                    if(isTRUE(all.equal(positions[i],positions[i-1]+1))) {
+                    if(positions[i]==positions[i-1]+1) {
                         varcovc <- cbind(varcovc,rep(0,dim(summary(anaMrd)$cov.unscaled)[1]))
                     } else {
                         varcovc <- cbind(varcovc,summary(anaMrd)$cov.unscaled[,(positions[i-1]-i+2):(positions[i]-i)],rep(0,dim(summary(anaMrd)$cov.unscaled)[1]))
@@ -269,11 +272,11 @@
             varcovc <- cbind(varcovc,summary(anaMrd)$cov.unscaled[,(positions[length(positions)]-length(positions)+1):dim(summary(anaMrd)$cov.unscaled)[2]])
             # Insertion de lignes de zros
             varcov <- rbind(varcovc[1:(positions[1]-1),],rep(0,dim(varcovc)[2]))
-            if(isTRUE(length(positions)>1))
+            if(length(positions)>1)
             {
                 for ( i in 2:length(positions))
                 {
-                    if(isTRUE(all.equal(positions[i],positions[i-1]+1))) {
+                    if(positions[i]==positions[i-1]+1) {
                         varcov <- rbind(varcov,rep(0,dim(varcovc)[2]))
                     } else {
                         varcov <- rbind(varcov,varcovc[(positions[i-1]-i+2):(positions[i]-i),],rep(0,dim(varcovc)[2]))
@@ -300,16 +303,16 @@
         beta <- Beta
         for (i in (1:I))
         {
-                if (identical(vm[i],"none")) # cas du no model
+                if (vm[i]=="none") # cas du no model
                 {
                         pstar[i]<- exp(beta[1]+log(2^vt[i]-1))/(1+exp(beta[1]+log(2^vt[i]-1)))
-                        dpstar[(2*I+ifelse(i>1,sum(Xw$nbparam[1:(i-1)]),0)):(2*I-1+sum(Xw$nbparam[1:i])),i] <- exp(beta[1]+log(2^vt[i]-1))/(1+exp(beta[1]+log(2^vt[i]-1)))^2
+                        dpstar[(2*I + if(i>1) sum(na.rm=TRUE,Xw$nbparam[1:(i-1)]) else 0):(2*I-1+sum(na.rm=TRUE,Xw$nbparam[1:i])),i] <- exp(beta[1]+log(2^vt[i]-1))/(1+exp(beta[1]+log(2^vt[i]-1)))^2
                         beta <- beta[-1]
                 } else # Tous les autres modles
                 {
-                        Xpf <- Xclosedp(vt[i],vm[i],vh[[i]],va[i])
-                        pstar[i] <- sum(exp(Xpf$mat%*%beta[c(1:Xpf$nbparam)]))/(1+ sum(exp(Xpf$mat%*%beta[c(1:Xpf$nbparam)])))
-                        dpstar[(2*I+ifelse(i>1,sum(Xw$nbparam[1:(i-1)]),0)):(2*I-1+sum(Xw$nbparam[1:i])),i] <- t(Xpf$mat)%*%exp(Xpf$mat%*%beta[c(1:Xpf$nbparam)])/(1+sum(exp(Xpf$mat%*%beta[c(1:Xpf$nbparam)])))^2
+                        Xpf <- Xclosedp(vt[i],vm[i],vh[i],va[i])
+                        pstar[i] <- sum(na.rm=TRUE,exp(Xpf$mat%*%beta[c(1:Xpf$nbparam)]))/(1+ sum(na.rm=TRUE,exp(Xpf$mat%*%beta[c(1:Xpf$nbparam)])))
+                        dpstar[(2*I + if(i>1) sum(na.rm=TRUE,Xw$nbparam[1:(i-1)]) else 0):(2*I-1+sum(na.rm=TRUE,Xw$nbparam[1:i])),i] <- t(Xpf$mat)%*%exp(Xpf$mat%*%beta[c(1:Xpf$nbparam)])/(1+sum(na.rm=TRUE,exp(Xpf$mat%*%beta[c(1:Xpf$nbparam)])))^2
                         beta <- beta[-c(1:Xpf$nbparam)]
                 }
         }
@@ -376,11 +379,11 @@
         
         for ( i in 1:(I-1))
         {
-            if(isTRUE(all.equal(i,I-1)))
+            if(i==I-1)
             {
                 dphi[,i] <- -(phi[i]^2)*duv[,I-i+1]
             } else {
-                dphi[,i] <- -(phi[i]^2)*(duv[,I-i+1]*sum(uv[1:(I-i)])-uv[I-i+1]*rowSums(duv[,1:(I-i)]))/sum(uv[1:(I-i)])^2
+                dphi[,i] <- -(phi[i]^2)*(duv[,I-i+1]*sum(na.rm=TRUE,uv[1:(I-i)])-uv[I-i+1]*rowSums(duv[,1:(I-i)]))/sum(na.rm=TRUE,uv[1:(I-i)])^2
             }
         }
         varcovphi <- t(dphi)%*%varcov%*%dphi
@@ -412,10 +415,10 @@
         Npop[2:I]<-Npop[1]*cumprod((vv[2:I]/cumsum(vv[1:(I-1)])+1)*phi)
         for (i in 2:I)
         {
-            if(isTRUE(all.equal(i,2))) {
+            if(i==2) {
                 dNpop[,i] <- phi[i-1]*(vv[i]+1)*dNpop[,i-1] + Npop[i-1]*(vv[i]+1)*dphi[,i-1] + Npop[i-1]*phi[i-1]*dvv[,i]          
             } else {
-                dNpop[,i] <- phi[i-1]*(vv[i]/sum(vv[1:(i-1)])+1)*dNpop[,i-1] + Npop[i-1]*(vv[i]/sum(vv[1:(i-1)])+1)*dphi[,i-1] + Npop[i-1]*phi[i-1]*(sum(vv[1:(i-1)])*dvv[,i]-vv[i]*rowSums(dvv[,1:(i-1)]))/sum(vv[1:(i-1)])^2
+                dNpop[,i] <- phi[i-1]*(vv[i]/sum(na.rm=TRUE,vv[1:(i-1)])+1)*dNpop[,i-1] + Npop[i-1]*(vv[i]/sum(na.rm=TRUE,vv[1:(i-1)])+1)*dphi[,i-1] + Npop[i-1]*phi[i-1]*(sum(na.rm=TRUE,vv[1:(i-1)])*dvv[,i]-vv[i]*rowSums(dvv[,1:(i-1)]))/sum(na.rm=TRUE,vv[1:(i-1)])^2
             }
         }
         varcovtpop <- t(dNpop)%*%varcov%*%dNpop
@@ -437,7 +440,7 @@
 # Calcul du nombre total d'individus qui ont pass sur le territoire #
 #--------------------------------------------------------------------#
 
-        Ntot <- Npop[1]+sum(B)
+        Ntot <- Npop[1]+sum(na.rm=TRUE,B)
         dNtot <- dNpop[,1] + rowSums(dB)    
         NtotStderr <- sqrt(max(t(dNtot)%*%varcov%*%dNtot-Ntot,0))
 
@@ -450,8 +453,8 @@
         dimnames(modelfit) <- list("fitted model",c("deviance","    df","      AIC"))
         
         emigfit <- cbind(c(anaMrd2$deviance,anaMrd3$deviance),c(anaMrd2$df.residual,anaMrd3$df.residual),c(anaMrd2$aic,anaMrd3$aic))
-        if (isTRUE(all.equal(Inono,3))) { dimnames(emigfit) <- list(c("model with temporary emigration"),c("deviance","    df","      AIC"))
-        } else if (isTRUE(Inono>3)) { dimnames(emigfit) <- list(c("model with homogeneous temporary emigration","model with temporary emigration"),c("deviance","    df","      AIC"))}    
+        if (Inono==3) { dimnames(emigfit) <- list(c("model with temporary emigration"),c("deviance","    df","      AIC"))
+        } else if (Inono>3) { dimnames(emigfit) <- list(c("model with homogeneous temporary emigration","model with temporary emigration"),c("deviance","    df","      AIC"))}    
                
         titre.periode.emig<-rep(0,length(idpemig))
         for (i in 1:length(idpemig)){ titre.periode.emig[i]<-paste("period",idpemig[i])}
@@ -472,8 +475,8 @@
         uv <- cbind(uv,uvStderr)
         vv <- cbind(vv,vvStderr)
 
-        if (isTRUE(all.equal(Inono,3))) { dimnames(parap) <- list(titre.periode.emig,c("estimate","stderr")) 
-        } else if (isTRUE(Inono>3)) { dimnames(parap) <- list(c(titre.periode.emig,"homogenous"),c("estimate","stderr")) }
+        if (Inono==3) { dimnames(parap) <- list(titre.periode.emig,c("estimate","stderr")) 
+        } else if (Inono>3) { dimnames(parap) <- list(c(titre.periode.emig,"homogenous"),c("estimate","stderr")) }
         dimnames(pstar)<-list(titre.periode,c("estimate","stderr"))
         dimnames(phi)<-list(titre.inter.periode,c("estimate","stderr"))
         dimnames(Npop)<-list(titre.periode,c("estimate","stderr"))
@@ -501,7 +504,7 @@
         dimnames(covP)<-list(titre.P,titre.P)
 
      
-        ans<-list(n=sum(Y),models=models,model.fit=modelfit,emig.fit=emigfit,emig.param=parap,
+        ans<-list(n=sum(na.rm=TRUE,Y),models=models,model.fit=modelfit,emig.fit=emigfit,emig.param=parap,
                   capture.prob=pstar,survivals=phi,N=Npop,birth=B,Ntot=Ntot,
                   loglin.param=loglinearpara,u.vector=uv,v.vector=vv,cov=covP,neg=positions)
         class(ans) <- "robustd"
